// define PINS
//Table Stepper
#define TableStepperEnablePinPin       	2
#define TableStepperStepPinPin         	3
#define TableStepperDirectionPin		4
//Laser Stepper
#define LaserStepperEnablePin       	5
#define LaserStepperStepPin         	6
#define LaserStepperDirectionPin    	7
//Unused third stepper
#define ThirdStepperEnablePin 		   11
#define ThirdStepperStepPin            12
#define ThirdStepperDirectionPin       13
// unused fourth stepper
#define FourthStepperEnablePin         A0
#define FourthStepperStepPin           A1
#define FourthStepperDirectionPin      A2
// others
#define LightPin                       A3
#define LaserPin                       A4
#define MicroSteppingPin               A5  
  
//define command
#define DisableLaser                  100
#define EnableLaser                   101
 
#define RotateLaserCw            	  110
#define RotateLaserCcw           	  111

#define RotateTurnTableCw        	  120
#define RotateTurnTableCcw       	  121

#define EnableLight              	  130
#define DisableLight             	  131

#define Ping                     	  201
#define ACK                      	  202
#define NACK                     	  203
#define Error                    	  204

#define EnableMicroStepping		 	  141
#define DisableMicroStepping		  142

//define constants
#define TableStepper             		1
#define LaserStepper             		2

#define ClockWise                		1
#define CounterClockWise         		2

void setup() 
{ 
	Serial.begin(9600);
	pinMode(LaserPin, OUTPUT);
	pinMode(LightPin, OUTPUT);
	pinMode(MicroSteppingPin, OUTPUT);
	digitalWrite(MicroSteppingPin, HIGH);
	pinMode(TableStepperEnablePinPin, OUTPUT);
	pinMode(TableStepperDirectionPin, OUTPUT);
	pinMode(TableStepperStepPinPin, OUTPUT);
	pinMode(LaserStepperEnablePin, OUTPUT);
	pinMode(LaserStepperDirectionPin, OUTPUT);
	pinMode(LaserStepperStepPin, OUTPUT);
	pinMode(ThirdStepperEnablePin, OUTPUT);
	pinMode(ThirdStepperDirectionPin, OUTPUT);
	pinMode(ThirdStepperStepPin, OUTPUT);
	pinMode(FourthStepperEnablePin, OUTPUT);
	pinMode(FourthStepperDirectionPin, OUTPUT);
	pinMode(FourthStepperStepPin, OUTPUT);

	disableStepper(TableStepperEnablePinPin);  
	disableStepper(LaserStepperEnablePin);  
	disableStepper(ThirdStepperEnablePin);   
	disableStepper(FourthStepperEnablePin);  

	disableLight(); 
	disableLaser();
	Serial.write(ACK); 
} 

void loop() 
{
	if(Serial.available() > 0)
	{

		int incomingByte = Serial.read();
		int numberOfSteps = 0;
		int result = ACK;

                if(incomingByte == RotateLaserCw || incomingByte == RotateLaserCcw || incomingByte == RotateTurnTableCw || incomingByte == RotateTurnTableCcw || incomingByte == EnableLight)
                  delay(3); //otherwise second byte is read to fast = 0
                  
		switch(incomingByte)
		{    
			case DisableLaser:
				disableLaser();
				break;
			case EnableLaser:
				enableLaser();
				break;
			case RotateLaserCw:
				numberOfSteps = Serial.read();
				performSteps(LaserStepper, ClockWise, numberOfSteps);	   
				break;
			case RotateLaserCcw:
				numberOfSteps = Serial.read();
				performSteps(LaserStepper, CounterClockWise, numberOfSteps);	   
				break;
			case RotateTurnTableCw:
				numberOfSteps = Serial.read();          
				performSteps(TableStepper, ClockWise, numberOfSteps);	   
				break;
			case RotateTurnTableCcw:
				numberOfSteps = Serial.read();          
				performSteps(TableStepper, CounterClockWise, numberOfSteps);	   
				break;
			case EnableLight:
				incomingByte = Serial.read();
				enableLight(incomingByte);
				break;
			case DisableLight:
				disableLight();
				break;
			case EnableMicroStepping:
				enableMicroStepping();
				break;
			case DisableMicroStepping:
				disableMicroStepping();
				break;
			case Ping:
				delay(1);
				break;
			default:
				result = NACK;
				break; 
		} 

		Serial.write(result); 
	}
}

void performSteps(int stepper, int dir, int numberOfSteps)
{
  
	int stepperEnablePin = 0;
	int stepperDirectionPin = 0;
	int stepperStepPin = 0;
	
	switch(stepper)
	{
		case TableStepper:
			stepperEnablePin = TableStepperEnablePinPin;
			stepperDirectionPin = TableStepperDirectionPin;
			stepperStepPin = TableStepperStepPinPin;
			break;
		case LaserStepper:
			stepperEnablePin = LaserStepperEnablePin;
			stepperDirectionPin = LaserStepperDirectionPin;
			stepperStepPin = LaserStepperStepPin;
			break;
	}
	
        enableStepper(stepperEnablePin);
  //      delay(3);
	setStepperDirectionPin(stepperDirectionPin, dir);
//	delay(3);	
        for(int i=0; i<numberOfSteps; i++)
	{
		performStep(stepperStepPin);

	}
  //      delay(3);
	disableStepper(stepperEnablePin);

//delay(500);
}

void performStep(int stepperPin)
{
	//turn stepper on, sleep, turn stepper off
	
	digitalWrite(stepperPin, LOW);
	delay(3); 
	digitalWrite(stepperPin, HIGH);
	delay(3);
}

void setStepperDirectionPin(int stepperPin, int dir)
{
	switch(dir)
	{
		case ClockWise:
			digitalWrite(stepperPin, HIGH);
			break;
		case CounterClockWise:
			digitalWrite(stepperPin, LOW);
			break;
	}
}

void enableStepper(int stepperPin)
{
    digitalWrite(stepperPin, LOW);
}

void disableStepper(int stepperPin)
{
    digitalWrite(stepperPin, HIGH);
}

void disableLaser()
{
	digitalWrite(LaserPin, LOW);
}

void enableLaser()
{
	digitalWrite(LaserPin, HIGH);
}

void disableLight()
{
	digitalWrite(LightPin, LOW);
}

void enableLight(int hue)
{
	analogWrite(LightPin, hue);
}

void enableMicroStepping()
{
	digitalWrite(MicroSteppingPin, HIGH);
}

void disableMicroStepping()
{
	digitalWrite(MicroSteppingPin, LOW);
}
